----------------------------------------------------------------------------------
--      __  ____                 _   __                
--    /  |/  (_)_____________  / | / /___ _   ______ _
--   / /|_/ / / ___/ ___/ __ \/  |/ / __ \ | / / __ `/
--  / /  / / / /__/ /  / /_/ / /|  / /_/ / |/ / /_/ / 
-- /_/  /_/_/\___/_/   \____/_/ |_/\____/|___/\__,_/  
--                                                  
-- Copyright (c) 2019 MicroNova, LLC
-- www.micro-nova.com
--
-- Engineer: Jeremy Bowers
-- Create Date: 12/02/2019
-- Design Name: Logic Gates
-- Target Devices: Mercury 2
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;

entity sev_seg is
  port(
    -- Group1 - Inputs for the 7 segment logic
    clk_50mhz   : in std_logic;                         -- 50 MHz clock
    ze          : in std_logic_vector(3 downto 0);      -- gates4 results
    
    -- Group2 - Outputs to the baseboard 7 segment display
    ag_segments : out std_logic_vector(6 downto 0);     -- 7SEG_A through 7SEG_G.
    an          : out std_logic_vector(3 downto 0);     -- an selects the target
    dp          : out std_logic                         -- 7 segment display.
                                                        -- dp refers to 7SEG_DOT.
  );
end sev_seg;

architecture sev_seg of sev_seg is

    signal s : unsigned(1 downto 0) := (others => '0'); -- holds clock pulses to 
                                                        -- help with z digits
    signal result : std_logic := '0';                   -- cycled answers from
                                                        -- gates4 (pulled from ze)
    signal clkdiv : unsigned(19 downto 0) := (others => '0');
                                                        -- clock pulse counter
                                                        -- counts 50M per second

begin
    -- Clock divider (pulse counter)
    process(clk_50mhz)
    begin
       if clk_50mhz'event and clk_50mhz = '1' then      -- Each clock tick event,
           clkdiv <= clkdiv + 1;                        -- add to the counter.
       end if;
    end process;
    
    -- Follow 's' to determine when each 7 segment display should be active
    process(s)
    begin
       an <= "1111";
       if true then                 -- 7 seg multiplexing 
           an(to_integer(s)) <= '0';                    -- will occur at a rate 
       end if;                                          -- determined by s
    end process;     
    
    -- Sync up each logical result with its 7 segment display
    process(s, ze)
    begin
        case s is
            when "00" => 
                result <= ze(0);
            when "01" => 
                result <= ze(1);
            when "10" => 
                result <= ze(2);
            when others => 
                result <= ze(3);
        end case;
    end process;
    
    -- 7 segment decoder
    process(result)
    begin
        case result is
            when '0' => 
                ag_segments <= "1000000";               -- display '0'
            when '1' => 
                ag_segments <= "1111001";               -- display '1'
            when others => 
                ag_segments <= "1111111";               -- completely off
        end case;
    end process;
    
    -- Assignments
    s <= clkdiv(14 downto 13);                          -- clock sample
    dp <= '1';                                          -- decimal points off
    
end sev_seg;
